unit DataSetHelperUnit;

interface

uses
  DB;

type
  TDataSetEnumerationRecord = record
    DataSet: TDataSet;
  end;

type
  TDataSetEnumerator = class
  private
    FWeOpenedDataSet: Boolean;
    FDataSet: TDataSet;
    FFirstItem: Boolean;
    FEmpty: Boolean;
  public
    constructor Create(const DataSet: TDataSet);
    function GetCurrent: TDataSetEnumerationRecord;
    function MoveNext: Boolean;
    property Current: TDataSetEnumerationRecord read GetCurrent;
  end;

type
  TDataSetHelper = class helper for TDataSet
    public
function FieldByAnyName(const FieldNames: array of WideString): TField;
function GetEnumerator: TDataSetEnumerator;
end;

implementation

uses
  DBConsts, StringUtilsUnit;

{ TDataSetEnumerator }

constructor TDataSetEnumerator.Create(const DataSet: TDataSet);
begin
  inherited Create;
  Self.FDataSet := DataSet;
  if DataSet.Active then
    DataSet.First() // if it already was Open, make sure we are at Bof
  else
  begin
    DataSet.Open();
    FWeOpenedDataSet := True;
  end;
  Self.FFirstItem := True;
  Self.FEmpty := DataSet.Bof and DataSet.Eof;
end;

function TDataSetEnumerator.GetCurrent: TDataSetEnumerationRecord;
begin
  if Self.FEmpty then
    Result.DataSet := nil
  else
    Result.DataSet := FDataSet;
end;

function TDataSetEnumerator.MoveNext: Boolean;
begin
  // After opening a TDataSet, it always is at the first record,
  // but a TEnumerator expects to be *before* the first record
  // so we need to accommodate for that by the FFirstItem trick
  if Self.FEmpty then
    Result := False
  else
  begin
    if FFirstItem then
      FFirstItem := False
    else
      FDataSet.Next;
    Result := not FDataSet.Eof;
  end; // else if Self.FEmpty

  if not Result then
    if FWeOpenedDataSet then
      FDataSet.Close();
end;

function TDataSetHelper.FieldByAnyName(const FieldNames: array of WideString): TField;
var
  FieldName: WideString;
begin
  for FieldName in FieldNames do
  begin
    Result := FindField(FieldName);
    if Assigned(Result) then
      Exit;
  end;
  Result := nil; // prevent [DCC Warning] SqlQueryHelperUnit.pas(32): W1035 Return value of function 'TSQLQueryHelper.ParameterByAnyName' might be undefined
  DatabaseErrorFmt(SFieldNotFound, [CommaSeparated(FieldName)], Self);
end;

{ TDataSetHelper }

function TDataSetHelper.GetEnumerator: TDataSetEnumerator;
begin
  Result := TDataSetEnumerator.Create(Self);
end;

end.

